/************************************************************************
**
** FILE  : simstep.c
**
** ZWECK : 
**
** AUTOR : Michael C. Ancutici
**	   Universitaet Stuttgart, Fakultaet Informatik
**
** DATUM : 14.03.93
**									
*************************************************************************/
#include	"simstep.h"
#include	"sim.h"
#include	"misc.h"
#include	"ka.h"
#include	"tra.h"
#include	"ste.h"
#include	"celltype.h"
#include	"simbreak.h"




/*************************************************************************
** FUNKTION:	SimStepS
** ZWECK:	Fuehrt eine Schaltung (seriell) aus. 
** EINGABE:	TNr der schaltenden Transition.
** GLOBAL:	Markierung in Mark geaendert.
**		BreakList evtl. vergroessert.
** RETURN: 	NULL falls BreakList leer, sonst Zeiger auf erstes
**		Element.
** ANMERK: 	ACHTUNG:
**		In Mark m wird die negative Markierung der realen
**		Markierung M gehalten, d.h. m = -M.
**		neue Markierung N = f( M, S, T)
**			N =  M + ( d(ST) + d(TS) )	
**		       -N = -M - ( d(ST) + d(TS) )
**		        n =  m - ( d(ST) + d(TS) )	mit n = -N
**		Dabei ist d(ST) = Gewicht der ST-Kante und
**		d(TS) = Gewicht der TS-Kante mit negativem Vorzeichen,
**		d.h. d(TS) = -| d(TS) |
*************************************************************************/
OBJECT *SimStepS( TNr)
  int	TNr;
{
  register STELLE	**s;
  register int		*m;
  register KANTE	**k;
  int			i;
  int			add = 0;
  int			alt;
  char			bc;
  
  TransCond[ TNr] = GEFEUERT;
  
  k = &(KaKante[ 0][ TNr].NachTrans);
  for (i=0, m=Mark, s=SteStelle; i<STEMAX; i++, m++, s++) {
    /**** ST-KANTE **************************************************/
    if ( (*k) && !((*k)->NegativKante) )
      add = (*k)->delta;
    /**** TS-KANTE **************************************************/
    if ( *(k+1))
      add += (*(k+1))->delta;

    k += KaSDisp;
    /**** BREAKBEDINGUNG STELLE ERFUELLT ? **************************/
    if ( (add) ) {
      alt = *m;
      *m -= add;
      add = 0;
      if ( (bc=(*s)->BreakCond) != KEINE ) {
        if ( (*s)->BreakValue != ALTE_MARKE ) 
          alt = - (*s)->BreakValue;
        switch (bc) {
          case KLEINER:
            if (*m > alt)
              SimBreakAddBL( CE_STELLE, i);  
          break;
          case GROESSER:
            if (*m < alt)
              SimBreakAddBL( CE_STELLE, i);  
          break;
          case GLEICH:
            if (*m == alt)
              SimBreakAddBL( CE_STELLE, i);  
          break;
          case UNGLEICH:
            if (*m != alt)
              SimBreakAddBL( CE_STELLE, i);  
          break;
        }
      }  
    }
  }  
  
  /**** BREAKBEDINGUNG KANTE ERFUELLT ? *****************************/
  if ( (TraTrans[ TNr]->Break) )
    SimBreakAddBL( CE_TRANSITION, TNr);

  return (BreakList->next);
}




/*************************************************************************
** FUNKTION:	SimStepP
** ZWECK:	Fuehrt einen ParallelSCHRITT durch. Dieser besteht aus der 
**		sequentiellen Ausfuehrung von EinzelSCHALTUNGEN. Es schalten
**		nur diejenigen Transitionen, die sich im Zustand AKTIV
**		befinden.
** GLOBAL:	Markierung in Mark aktualisiert und evtl BreakList erweitert.
** RETURN: 	Zeiger auf erstes Breakelement in BreakList.
** ANMERK: 	Jetzt schalten alle aktiven Transitionen in der Reihenfolge
**		ihrer Nummerierung. In SimStepS wird nach jeder Schaltung auf
**		eine positive Breakbedingung getestet und die BreakList 
**		entsprechend ERWEITERT, d.h. nur hinzugefuegt. Wenn jetzt
**		aber der Fall eintritt, dass nach einer darauffolgenden
**		Schaltung die neue Markierung eine negative Breakbedingung
**		liefert, dann wird die zuvor in die BreakList eingetragene 
**		Stelle NICHT entfernt. SimStepP testet seinerseits nicht auf
**		Breaks nach jeder Schaltung, sondern laesst stur jede weitere
**		aktive Transition schalten. Somit kann nach einem folgendes
**		passieren:
**		
**		Vor SimStepP:
**		1. Break > 10 auf S2
**		2. (S1, 1) -> (T1) -10-> (S2, 1) -> (T2) -> (S3, 0)
**		
**		Nach SimStepP zwei moegliche Zustaende:
**		1. T2 schaltet:
**		   (S1, 1) -> (T1) -10-> (S2, 0) -> (T2) -> (S3, 1)
**		   T1 schaltet:
**		   (S1, 0) -> (T1) -10-> (S2, 10) -> (T2) -> (S3, 1)
**		
**		2. T1 schaltet
**		   (S1, 0) -> (T1) -10-> (S2, 11) -> (T2) -> (S3, 0)
**		   Breakbedingung >10 in S2 erfuellt.
**		   T2 schaltet:
**		   (S1, 0) -> (T1) -10-> (S2, 10) -> (T2) -> (S3, 1)
**		   	
**		Sowohl Fall 1 als auch Fall 2 liefern dieselbe Endmarkierung.
**		Nach Fall 2 aber enthaelt die BreakList S2, da nach Schalten
**		von T1 kurzfristig die Bedingung >10 erfuellt war. Wenn wir
**		uns aber die Endmarkierung betrachten, spricht keine Break
**		bedingung an. 
**
**		FAZIT:
**		Wenn nach einem Parallelschritt Breaks vorliegen, heisst das,
**		dass die entsprechenden Stellen (und nur Stellen) WAEHREND der
**		Ausfuehrung des Schrittes irgendeinmal der Breakbedingung
**		entsprochen haben.
**
**		ALTERNATIVEN:
**		1. Sofortiger Abbruch des Schrittes, falls eine Schaltung
**		   ein Break ausloest. Dann haben aber nicht alle Transitionen
**		   schalten lassen, die aktiv sind.
**		2. Breakelement aus BreakList wieder entfernen. Teuer.
**		   
*************************************************************************/
OBJECT *SimStepP()
{
  register int	i;
  register char	*c;
  
  for (i=0, c=TransCond; i<TRAMAX; i++, c++)
    if (*c==AKTIV)
      SimStepS( i);

  return (BreakList->next);
}




